package cytoPrefuse;

import java.awt.BasicStroke;
import java.awt.Color;

import cytoscape.visual.LineType;

import prefuse.Display;
import prefuse.Visualization;
import prefuse.action.ActionList;
import prefuse.action.RepaintAction;
import prefuse.action.assignment.ColorAction;
import prefuse.action.assignment.StrokeAction;
import prefuse.data.Table;
import prefuse.data.column.ColumnMetadata;
import prefuse.util.ColorLib;
import prefuse.util.StrokeLib;
import prefuse.visual.DecoratorItem;
import prefuse.visual.EdgeItem;
import prefuse.visual.NodeItem;
import prefuse.visual.VisualItem;

public class ColorManager {
	//red on white
	private final static int Highlight1 = ColorLib.hsb (0, 0.9f, 0.8f);
	private final static int Highlight1Inverse = ColorLib.hsb (0, 0.0f, 1.0f);

	//white on green
	private final static int Highlight2 = ColorLib.hsb (0.3f, 0.9f, 0.8f);
	private final static int Highlight2Inverse = ColorLib.hsb (0, 0.0f, 1.0f);
	
	//white on blue
	private final static int Highlight3 = ColorLib.hsb (0.65f, 0.9f, 0.8f);
	private final static int Highlight3Inverse = ColorLib.hsb (0, 0.0f, 1.0f);
	
		
	private final static float[] hsbVals = new float[3];
	
	public LabelColorAction labelColorAction = new LabelColorAction(CytoPrefuse.LABELS);
	public LabelBGColorAction labelBGColorAction = new LabelBGColorAction (CytoPrefuse.LABELS);
	public MyStrokeAction edgeStrokeAction = new MyStrokeAction(CytoPrefuse.EDGES);

	public boolean pathColoring  = false;
	private Visualization vis;
	
	private Color nodeCytoSelected;
	private Color edgeCytoSelected;
	public ColorManager() {
		nodeCytoSelected = ColorLib.getColor(Highlight3);
		edgeCytoSelected = ColorLib.getColor(Highlight3);
	
	}
	public void createColorRules(Visualization vis, LabelLayout labelLayout, Table edges) {
		
		  
			FillNodeAction fillNodeAction = new FillNodeAction(CytoPrefuse.NODES);
			BorderAction borderAction = new BorderAction(CytoPrefuse.NODES);

			// create an action list containing all color assignments
			ActionList color = new ActionList();
			color.add(fillNodeAction);		
			color.add(borderAction);
			color.add(edgeStrokeAction);
			color.add(new EdgeFadeColorAction(CytoPrefuse.EDGES, edges));
			color.add(labelLayout);
			color.add(new RepaintAction());			
			vis.putAction("color", color);
			this.vis = vis;
	}
	
	public class FillNodeAction extends ColorAction {
		private FillNodeAction(String group){
			super (group, VisualItem.FILLCOLOR);
		}
		public int getColor (VisualItem item){
			NodeItem node = (NodeItem) item;
            if (node.isInGroup(CytoPrefuse.SELECTED)){
                return ColorLib.color(nodeCytoSelected);
			}
		   Color cytoColor = (Color) node.get(CytoPrefuse.FILL_COLOR);
		   if (cytoColor==null){
			   return 0;
		   }
		   return ColorLib.color(cytoColor);
		  
		}
			
	}
	
	public  class BorderAction extends ColorAction {
		private BorderAction(String group){
			super (group, VisualItem.STROKECOLOR);
		}
		public int getColor(VisualItem item) {
			NodeItem node = (NodeItem) item;
			if (node.isInGroup(CytoPrefuse.SINGLE_HIGHLIGHT)) {
				return Highlight2;
			}
			if (node.getBoolean(VisualItem.HIGHLIGHT)){
				return Highlight1;					
			}			
                
            if (node.isInGroup(CytoPrefuse.SELECTED)){
               return ColorLib.color(nodeCytoSelected);
			}
			if (node.getBoolean(VisualItem.HOVER)) {
				return Highlight1;
			}
			
			Color c = (Color) item.get(CytoPrefuse.BORDER_COLOR);
			if (c == null){
				return ColorLib.gray(60);
			}
			return ColorLib.color (c);
		}
		
	}
	public  class LabelColorAction extends ColorAction {	
		private LabelColorAction(String group) {
            super(group, VisualItem.TEXTCOLOR);
        }
	
		public int getColor(VisualItem item) {
			DecoratorItem di = (DecoratorItem) item;
			NodeItem node = (NodeItem) di.getDecoratedItem();
			
			if (node.isInGroup(CytoPrefuse.SINGLE_HIGHLIGHT)) {
				return Highlight2Inverse;
			}
			if (node.getBoolean(VisualItem.HIGHLIGHT)){
				return Highlight1Inverse;					
			}
            if (node.isInGroup(CytoPrefuse.SELECTED))
            	return Highlight3Inverse;
		
			if (node.getBoolean(VisualItem.HOVER)) {
				return Highlight1Inverse;
			}
			
			Color c = (Color) node.get(CytoPrefuse.TEXT_COLOR);		
			if (c == null){
				return ColorLib.gray(60);
			}
			return ColorLib.color(c);			
		
		}
	}
	
	private int changeSaturationTo (Color c, float saturation){
		if (c.getRed() == c.getGreen() &&
				c.getGreen() == c.getBlue()){
			//a grey scale.  Change brightness instead of saturation
			return ColorLib.gray(255-(int)(saturation *255));
		}else{
			Color.RGBtoHSB(c.getRed(),c.getGreen(), c.getBlue(), hsbVals);				
			return ColorLib.hsb(hsbVals[0], saturation, hsbVals[2]);
		}
	}
	
	public static int increasedContrast (int color) {
		Color c = ColorLib.getColor(color);
		Color.RGBtoHSB(c.getRed(),c.getGreen(), c.getBlue(), hsbVals);
		float deeperSaturation = hsbVals[1] * 1.2f;
		if (deeperSaturation > 1.0f){
			deeperSaturation = 1.0f;
		}
		float lessBright = hsbVals[2] * 0.8f;
		if (lessBright < 0.0f){
			lessBright = 0.0f;
		}
		return ColorLib.hsb(hsbVals[0],deeperSaturation, lessBright);
	}
	
	public  class LabelBGColorAction extends ColorAction {
		private LabelBGColorAction(String group) {
            super(group, VisualItem.FILLCOLOR);
        }
	
		public int getColor(VisualItem item) {
			DecoratorItem di = (DecoratorItem) item;
			NodeItem node = (NodeItem) di.getDecoratedItem();
			if (node.isInGroup(CytoPrefuse.SINGLE_HIGHLIGHT)) {
				return Highlight2;
			}
			if (node.getBoolean(VisualItem.HIGHLIGHT))
				return Highlight1;					
            
            if (node.isInGroup(CytoPrefuse.SELECTED))
                return ColorLib.color(nodeCytoSelected);
					
	
			if (node.getBoolean(VisualItem.HOVER)) {
				return Highlight1;		
			}
			Color bgColor;
			
			if (vis != null){
				Display d = vis.getDisplay(0);
				bgColor= d.getBackground();
			}
			else
				bgColor = ColorLib.getGrayscale(0);
		    
			return ColorLib.rgba(bgColor.getRed(), bgColor.getGreen(), bgColor.getBlue(), 150);		    
								
		}		
	}
	
	public  class MyStrokeAction extends StrokeAction {
	
		private MyStrokeAction( String group){
			super (group);
		}
		public BasicStroke getStroke( VisualItem item){
			
			
			float scale =2.0f;
			
			LineType lt =(LineType) item.get(CytoPrefuse.LINE_TYPE);
			BasicStroke basicStroke;
			if (lt != null){
				basicStroke = (BasicStroke)lt.getStroke();
			}else{
				basicStroke =StrokeLib.getStroke(scale, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND);
			}
			scale *= basicStroke.getLineWidth();
			if (item.isInGroup(CytoPrefuse.SINGLE_HIGHLIGHT)){
				return StrokeLib.getDerivedStroke(basicStroke, 3 * scale);
			}else if (item.getBoolean(VisualItem.HOVER)) {
				return StrokeLib.getDerivedStroke(basicStroke, 2*scale);
			}else if (item.getBoolean(VisualItem.HIGHLIGHT)){
				return StrokeLib.getDerivedStroke(basicStroke,2*scale);
			}else if (item.isInGroup(CytoPrefuse.SELECTED)){
				return StrokeLib.getDerivedStroke(basicStroke,2*scale);
			}else {
				return StrokeLib.getDerivedStroke(basicStroke,1*scale);
			}			
			
		}
	}
	public class EdgeFadeColorAction extends ColorAction{		
		
		private Table edges;
		public EdgeFadeColorAction(String group, Table edges) {
            super(group, VisualItem.STROKECOLOR);
            this.edges = edges;
		}
		
		public int getColor(VisualItem item) {
			if (item.isInGroup(CytoPrefuse.SINGLE_HIGHLIGHT))
				return Highlight2;
			if (item.getBoolean(VisualItem.HOVER)) 
				return Highlight1;
			if (item.getBoolean(VisualItem.HIGHLIGHT))
				return Highlight1;					
            if (item.isInGroup(CytoPrefuse.SELECTED))
                return ColorLib.color(edgeCytoSelected);					
			
			ColumnMetadata edgeLengthMetadata = edges.getMetadata(CytoPrefuse.EDGE_LENGTH);
			int maxRow =edgeLengthMetadata.getMaximumRow();					
			float maxLength = edges.getInt(maxRow, CytoPrefuse.EDGE_LENGTH);
			
		    EdgeItem   edge = (EdgeItem)item;	 
		 	 Color c = (Color)edge.get(CytoPrefuse.FILL_COLOR);
		 	 if (c == null){
		 		 c = ColorLib.getColor(255,255,255);
		 	 }
	 	 
	 	 	 int edgeLength = edge.getInt(CytoPrefuse.EDGE_LENGTH);
		     double normalizedLength= 1- (edgeLength / maxLength);
	    	 double alphaFactor = normalizedLength* normalizedLength * normalizedLength * normalizedLength;
		
	    	 return changeSaturationTo(c,  (float) ((100 + 155*alphaFactor)/255.0f));
	    
		}
	}

	public boolean isSpecialColoring(EdgeItem item){
		if (item.isInGroup(CytoPrefuse.SINGLE_HIGHLIGHT))
			return true;
		if (item.getBoolean(VisualItem.HOVER)) 
			return true;
		if (item.getBoolean(VisualItem.HIGHLIGHT))
			return true;					
		if (item.isInGroup(CytoPrefuse.SELECTED))
			return true;
		
		return false;
		
	}
	public void setCytoNodeSelected(Color nodeSelectionColor) {
		if (nodeSelectionColor != null)
			nodeCytoSelected = nodeSelectionColor;
	}

	public void setCytoEdgeSelected(Color edgeSelectionColor) {
		if (edgeSelectionColor != null)
			edgeCytoSelected = edgeSelectionColor ;
	}

}
