package cytoPrefuse;
import java.awt.event.ActionEvent;
import java.awt.geom.Rectangle2D;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Iterator;

import javax.swing.JOptionPane;
import javax.swing.ProgressMonitor;
import javax.swing.SwingConstants;

import prefuse.Visualization;
import prefuse.util.GraphicsLib;
import prefuse.util.display.DisplayLib;
import prefuse.util.display.ExportDisplayAction;
import cytoscape.CyNetwork;
import cytoscape.Cytoscape;
import cytoscape.data.CyAttributes;
import cytoscape.plugin.CytoscapePlugin;
import cytoscape.util.CytoscapeAction;

public class PluginEntry extends CytoscapePlugin implements PropertyChangeListener {

		/**
	 * This class gets attached to the menu item.
	 */
	

	public class CreateCellLayout extends CytoscapeAction {
	
		private static final long serialVersionUID = 1L;
	
		public static final String CREATE_MENU_TEXT ="Create Cerebral view"; 
		/**
		 * The constructor sets the text that should appear on the menu item.
		 */
		public CreateCellLayout() {
			super (CREATE_MENU_TEXT);
      }
	
		/**
		 * This method is called when the user selects the menu item.
		 */
		public void actionPerformed(ActionEvent ae) {
	
			CyNetwork cyNetwork = Cytoscape.getCurrentNetwork();
			if (cyNetwork.getNodeCount() == 0){
				String message = "Choose a network in the network panel before using this menu option.";
				JOptionPane.showMessageDialog(Cytoscape.getDesktop(), message);				    
				return;
			}
			if (Cytoscape.getNodeAttributes().getAttributeNames().length == 0){
				String message = "The nodes in the current network have no attributes.\n  Please import cell localization node attributes before using this menu option.";
				JOptionPane.showMessageDialog(Cytoscape.getDesktop(), message);				    
				return;
			}
			
			CreateViewDialog cvd = new CreateViewDialog();
			CyAttributes globalAttributes = Cytoscape.getNetworkAttributes();
			String t=globalAttributes.getStringAttribute(Cytoscape.getCurrentNetwork().getIdentifier(), "cerebralParameters");
			if (t != null && !t.equals("")){
				cvd.loadFromString(t);
			}			
			cvd.showDialog();
			if (cvd.okToRender){
		//		CytoscapeLayoutMonitor layoutMonitor = new CytoscapeLayoutMonitor(cvd);
//				
				ProgressMonitor progressMonitor = new ProgressMonitor(Cytoscape.getDesktop().getNetworkViewManager().getDesktopPane(),
"Creating layout.", "", 0, 1000);
				progressMonitor.setMillisToDecideToPopup(100);
				progressMonitor.setMillisToPopup(400);
				progressMonitor.setNote("Estimated time left 1000 minutes.");
			
				CytoPrefuse cytoPrefuse = new CytoPrefuse(cvd, false);
				
				cytoPrefuse.searchGridLayout.setProgressMonitor (progressMonitor);
				Visualization vis= cytoPrefuse.getVisualization();			
				vis.runAfter("layout", "layoutPostProcess");
				vis.run ("layout");
				vis.run("applyVizMapper");
			}
		}

		
	
	}
	public class ShowCellLayout extends CytoscapeAction {
	
		private static final String RESTORE_MENU = "Restore previous Cerebral views";
		private static final long serialVersionUID = -1304975942931876614L;
	
		public ShowCellLayout() {
			super(RESTORE_MENU);
		}
	
		public void actionPerformed(ActionEvent arg0) {
			Iterator networksIt = Cytoscape.getNetworkSet().iterator();
			boolean atLeastOneLoaded = false;
			while (networksIt.hasNext()) {
				CyNetwork network = (CyNetwork) networksIt.next();
				Cytoscape.setCurrentNetwork(network.getIdentifier());
				CyAttributes globalAttributes = Cytoscape.getNetworkAttributes();
				String t=globalAttributes.getStringAttribute(Cytoscape.getCurrentNetwork().getIdentifier(), "cerebralParameters");
	
				if (t != null && !t.equals("")){
					CreateViewDialog cvd = new CreateViewDialog();
					cvd.loadFromString(t);
					CytoPrefuse cytoPrefuse = new CytoPrefuse(cvd, true);					
				//	cytoPrefuse.loadNetwork();
					Visualization vis= cytoPrefuse.getVisualization();			
					vis.run("applyVizMapper");
				
					atLeastOneLoaded = true;
				}
				
			}
			if (!atLeastOneLoaded){
				  String message = "This session does not have any saved Cerebral layouts.\nCreate a layout by choosing " + CreateCellLayout.CREATE_MENU_TEXT + " from the menu.";
				  JOptionPane.showMessageDialog(Cytoscape.getDesktop(), message);
			}
		}
		
	}
    
    public class ExportView extends CytoscapeAction{
        private static final long serialVersionUID = 1L;
        private static final String EXPORTVIEW_MENU = "Export Cerebral view";
        public ExportView(){
            super(EXPORTVIEW_MENU);
        }
        
        public void actionPerformed(ActionEvent arg0) {
            CytoPrefuse currentView = ControlPanel.instance().getCurrentVis();
            if (currentView != null){
                ExportDisplayAction eda = new ExportDisplayAction(currentView);
                eda.actionPerformed(new ActionEvent(this, 0, ""));
            }
            
        }
    }
    
	private static final String PLUGINS = "Plugins";
	public PluginEntry(){
//		 create a new action to respond to menu activation
		CreateCellLayout action = new CreateCellLayout();
		// set the preferred menu
		action.setPreferredMenu(PLUGINS);
		// and add it to the menus

		ShowCellLayout showCellLayout = new ShowCellLayout();
		showCellLayout.setPreferredMenu(PLUGINS);
        
        ExportView exportView =  new ExportView();
        exportView.setPreferredMenu(PLUGINS);

		Cytoscape.getDesktop().getCyMenus().addAction(action);
		Cytoscape.getDesktop().getCyMenus().addAction(showCellLayout);
		Cytoscape.getDesktop().getCyMenus().addAction(exportView);
        
		//Add our control panel
		Cytoscape.getDesktop().getCytoPanel(SwingConstants.WEST).add("Cerebral",null, ControlPanel.instance(),"Cerebral");
		
		Cytoscape.getSwingPropertyChangeSupport().addPropertyChangeListener(Cytoscape.SESSION_LOADED, this);
        Cytoscape.getSwingPropertyChangeSupport().addPropertyChangeListener(Cytoscape.CYTOSCAPE_INITIALIZED, new JavaWSLauncher());
        
	}
    
    private class JavaWSLauncher implements PropertyChangeListener{

        public void propertyChange(PropertyChangeEvent evt) {
            if (System.getProperty("javawslaunch")!= null){
                String parameters = "version1.1=The Label=Localization=interaction=true=Function=2=Cytoplasm=1=Plasma membrane=3=Nucleus=0=Extracellular==unknown=true=Cytoplasm=false=Plasma membrane=false=Nucleus=false=Extracellular=false==Cytoplasm=2=Plasma membrane=1=Nucleus=3=Extracellular=0==pp=false=pd=true==";
               
                CreateViewDialog cvd = new CreateViewDialog();
                cvd.loadFromString(parameters);
                ProgressMonitor progressMonitor = new ProgressMonitor(Cytoscape.getDesktop().getNetworkViewManager().getDesktopPane(),
                        "Creating layout.", "", 0, 1000);
                progressMonitor.setMillisToDecideToPopup(100);
                progressMonitor.setMillisToPopup(400);
                progressMonitor.setNote("Estimated time left 1000 minutes.");
            
                CytoPrefuse cytoPrefuse = new CytoPrefuse(cvd, false);
                
                cytoPrefuse.searchGridLayout.setProgressMonitor (progressMonitor);
                Visualization vis= cytoPrefuse.getVisualization();          
                vis.run ("layout");
                vis.run("applyVizMapper");                   
            }  
        }
    }

	/**
	 * Gives a description of this plugin.
	 */
	public String describe() {
		return "";
	}

	public void propertyChange(PropertyChangeEvent e) {
        if (e.getPropertyName().compareToIgnoreCase("SESSION_LOADED")==0){
            Iterator networksIt = Cytoscape.getNetworkSet().iterator();
            while (networksIt.hasNext()) {
    			CyNetwork network = (CyNetwork) networksIt.next();
    			Cytoscape.setCurrentNetwork(network.getIdentifier());
    			CyAttributes globalAttributes = Cytoscape.getNetworkAttributes();
    			String t=globalAttributes.getStringAttribute(Cytoscape.getCurrentNetwork().getIdentifier(), "cerebralParameters");
    
    			if (t != null && !t.equals("")){
    				CreateViewDialog cvd = new CreateViewDialog();
    				cvd.loadFromString(t);
    				CytoPrefuse cytoPrefuse = new CytoPrefuse(cvd, true);					
    				//cytoPrefuse.loadNetwork();
    				Visualization vis= cytoPrefuse.getVisualization();			
    				vis.run("applyVizMapper");
    				
    				 vis.getDisplay(0).setHighQuality(false);
    				    
    			     Rectangle2D bounds = vis.getBounds(CytoPrefuse.NODES);
    		         GraphicsLib.expand(bounds, 50 + (int)(1/cytoPrefuse.getScale()));
    		         DisplayLib.fitViewToBounds(cytoPrefuse, bounds, 2000);
    		         vis.runAfter("delayHighQuality", 2000);
                     
    			}			
    		}
        }
	}




}
