package prefuse.render;

import java.awt.Graphics2D;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;

import prefuse.Display;
import prefuse.visual.EdgeItem;
import prefuse.visual.NodeItem;
import prefuse.visual.VisualItem;
import cytoPrefuse.CytoPrefuse;

//Just like ShapeRenderer except we'll trigger mouse over events to an area twice as
//big as our object

//Also draws the starts of lines that join to this object over the object border in
// the color of this node.
public class EasyHoverShapeRenderer extends ExtendedShapeRenderer {
	 

	
	protected Line2D       m_line  = new Line2D.Float();	
	
	  private Display d;
	  private double scaleRange;
	  private double minScale;
	  private BundleEdgeRenderer bundleEdgeRenderer;
      public EasyHoverShapeRenderer( Display d) {
		  this.d = d;
          bundleEdgeRenderer = new BundleEdgeRenderer();
		 
		  Rectangle2D bounds = d.getVisualization().getBounds(CytoPrefuse.NODES);
		  double w = d.getWidth();
		  double h = d.getHeight();
          double cx = bounds.getCenterX();
          double cy =  bounds.getCenterY();
          
          // compute half-widths of final bounding box around
          // the desired center point
          double wb = Math.max(cx-bounds.getMinX(),
                               bounds.getMaxX()-cx);
          double hb = Math.max(cy-bounds.getMinY(),
                               bounds.getMaxY()-cy);
          
          // compute scale factor
          //  - figure out if z or y dimension takes priority
          //  - then balance against the current scale factor
          minScale = Math.min(w/(2*wb),h/(2*hb));
          scaleRange = 1.5 - minScale;
	  }
	  public void setBounds(VisualItem item) {
		    super.setBounds(item);
		    Rectangle2D bounds = (Rectangle2D) item.get(VisualItem.BOUNDS);
		    double extraFactor =  1- ( (d.getScale() - minScale) / scaleRange);
		    if (extraFactor < 0)
		    	extraFactor = 0;
		    if (extraFactor> 1)
		    	extraFactor =1;
		    
		    double extraWidth = bounds.getWidth() *extraFactor;
		    double extraHeight = bounds.getHeight() *extraFactor;
		    bounds.setRect( bounds.getX() - extraWidth,
		    				bounds.getY() - extraHeight,
		    				bounds.getWidth() + 2*(extraWidth),
		    				bounds.getHeight() + 2 *(extraHeight));
		    
	    }
	   public boolean locatePoint(Point2D p, VisualItem item) {
	        if ( item.getBounds().contains(p) ) {
	        	return true;
	        }
	        return false;
	   }
	
	   public void render(Graphics2D g, VisualItem item) {		  	 
		   super.render(g,item);
		
           List sortedEdges = new ArrayList();
		   RenderSorter renderSorter = new RenderSorter();
		   //Now draw coloured join lines over the edge of the shape
		   if (item instanceof NodeItem) {
			  NodeItem node = (NodeItem)item;
              
              Rectangle2D bounds = item.getBounds();
              Rectangle2D expandedBounds = new Rectangle2D.Double(bounds.getX()-3.0, bounds.getY() -3.0, bounds.getWidth() + 6.0, bounds.getHeight() + 6.0);
              g.setClip(expandedBounds);
		
			  sortedEdges.clear();
			  Iterator edgesIt = node.edges();
		      while (edgesIt.hasNext()) {
		    	  sortedEdges.add(edgesIt.next());
		      }
		      Collections.sort(sortedEdges, renderSorter);
		     
		      edgesIt = sortedEdges.iterator();
		      while (edgesIt.hasNext()) {
		    	  EdgeItem edge = (EdgeItem) edgesIt.next();                  
                  bundleEdgeRenderer.render(g,edge);
              }
              g.setClip(null);			           		  
		   }
           
           
	   }
	   
	   private class RenderSorter implements Comparator {

			public int compare(Object arg0, Object arg1) {
				EdgeItem e1 = (EdgeItem)arg0;
				EdgeItem e2 = (EdgeItem)arg1;
				if (e1.isInGroup(CytoPrefuse.SINGLE_HIGHLIGHT))
					return 1;
				if (e2.isInGroup(CytoPrefuse.SINGLE_HIGHLIGHT))
					return -1;
							
					
					if (e1.getBoolean(VisualItem.HOVER)) 
							return 1;
					if (e2.getBoolean(VisualItem.HOVER)) 
							return -1;
							
					if (e1.getBoolean(VisualItem.HIGHLIGHT))
							return 1;
					if (e2.getBoolean(VisualItem.HIGHLIGHT))
							return -1;								
					if (e1.isInGroup(CytoPrefuse.SELECTED))
							return 1;	
					if (e2.isInGroup(CytoPrefuse.SELECTED))
						return -1;	
				
					if (e1.getInt(CytoPrefuse.EDGE_LENGTH) < e2.getInt(CytoPrefuse.EDGE_LENGTH)){
						return 1;
					}else				
						return -1;		
			}

		}
}
